package com.tzld.piaoquan.wecom.job;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.wecom.dao.mapper.UserMapper;
import com.tzld.piaoquan.wecom.model.bo.Miniprogram;
import com.tzld.piaoquan.wecom.model.po.HistoryMessage;
import com.tzld.piaoquan.wecom.service.AccessTokenService;
import com.tzld.piaoquan.wecom.service.HistoryMessageService;
import com.tzld.piaoquan.wecom.utils.HttpClientUtil;
import com.tzld.piaoquan.wecom.utils.HttpPoolClient;
import com.tzld.piaoquan.wecom.utils.URLUtil;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
public class WeComHistoryDataJob {

    @Autowired
    private AccessTokenService accessTokenService;


    String GET_HISTORY_DATA_URL = "https://open.weibanzhushou.com/open-api/group_msg/result";

    String ACCESS_TOKEN = "b41281675156a3355c2f4d61398a24d3";

    private static final HttpPoolClient httpPoolClientDefault = HttpClientUtil.create(30000, 30000, 2000, 5000, 5, 30000);

    final int size = 20;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private HistoryMessageService historyMessageService;

    public void selectHistoryMessageByDay(){
        long startTime = 1726416000L;
        long endTime = System.currentTimeMillis() / 1000;
        for (;startTime < endTime; startTime += 60 * 60 * 24){
            selectHistoryMessage(startTime, startTime + 60 * 60 * 24);
        }
    }

    public void selectHistoryMessage(Long startTime, Long endTime) {
        try {
            //TODO accessToken 暂时写死 后续增加接口刷新和存储
            String res = getHistoryData(size, 0, startTime, endTime);
            //TODO 记录查询数据 info日志
            if (ObjectUtils.isEmpty(res)) {
                return;
            }
            JSONObject jsonObject = JSONObject.parseObject(res);
            JSONArray jsonArray = jsonObject.getJSONArray("missions");
            for (int i = 0; i < jsonArray.size(); i++) {
                List<String> allExternalUserList = new ArrayList<>();
                List<Miniprogram> miniprogramList = new ArrayList<>();
                Date createdAt = jsonArray.getJSONObject(i).getDate("created_at");
                JSONArray attachments = jsonArray.getJSONObject(i).getJSONObject("msg_data").getJSONArray("attachments");
                for (int j = 0; j < attachments.size(); j++) {
                    Miniprogram miniprogram = new Miniprogram();
                    String page = attachments.getJSONObject(j).getJSONObject("miniprogram").getString("page");
                    //历史先用切割法
                    String translate = URLUtil.translate(page);
                    String videoId = translate.split("id=")[1].split("&")[0];
                    if (StringUtils.isNotEmpty(videoId) && StringUtils.isNumeric(videoId)) {
                        miniprogram.setVideoId(Long.parseLong(videoId));
                    }
                    miniprogram.setAttachmentIdx(j + 1);
                    miniprogramList.add(miniprogram);
                }
                JSONArray subMissionList = jsonArray.getJSONObject(i).getJSONArray("sub_mission_list");
                for (int k = 0; k < subMissionList.size(); k++) {
                    if (!"sended".equals(subMissionList.getJSONObject(k).getString("status"))) {
                        continue;
                    }
                    List<String> externalUserList = subMissionList.getJSONObject(k).getJSONArray("external_user_list").toJavaList(String.class);
                    if (CollectionUtils.isEmpty(externalUserList)) {
                        continue;
                    }
                    allExternalUserList.addAll(externalUserList);
                }
                insertHistoryMessageList(allExternalUserList, miniprogramList, createdAt);
            }
        } catch (IOException e) {
            //TODO 记录异常日志 记录起止时间和页数 startTime endTime i
            e.printStackTrace();
        }
    }


    private void insertHistoryMessageList(List<String> allExternalUserList, List<Miniprogram> miniprogramList, Date sendTime) {
        List<HistoryMessage> historyMessageList = new ArrayList<>();
        for (String externalUserId3rdParty : allExternalUserList) {
            Long userId = userMapper.selectIdByExternalUserId3rdParty(externalUserId3rdParty);
            if (userId == null) {
                continue;
            }
            for (Miniprogram miniprogram : miniprogramList) {
                HistoryMessage historyMessage = new HistoryMessage();
                historyMessage.setCreateTime(sendTime);
                historyMessage.setAttachmentIdx(miniprogram.getAttachmentIdx());
                historyMessage.setVideoId(miniprogram.getVideoId());
                historyMessage.setUserId(userId);
                historyMessageList.add(historyMessage);
            }
        }
        historyMessageService.batchInsertHistoryMessage(historyMessageList);
    }

    public Integer getHistoryDataTotal(Long startTime, Long endTime) throws IOException {
        String res = getHistoryData(1, 0, startTime, endTime);
        JSONObject jsonObject = JSONObject.parseObject(res);
        return jsonObject.getInteger("total");
    }

    public String getHistoryData(Integer limit, Integer offset, Long startTime, Long endTime) throws IOException {
        String accessToken = accessTokenService.getAccessToken();
        String url = GET_HISTORY_DATA_URL
                + "?access_token=" + accessToken
                + "&limit=" + limit + "&offset=" + offset + "&start_time=" + startTime + "&end_time=" + endTime;
        return httpPoolClientDefault.get(url);
    }
}






